# This file is auto-generated by the Perl DateTime Suite time zone
# code generator (0.07) This code generator comes with the
# DateTime::TimeZone module distribution in the tools/ directory
#
# Generated from ../../data/tz/Olson/asia.  Olson data version 2003d
#
# Do not edit this file directly.
#
package DateTime::TimeZone::Asia::Kuala_Lumpur;

use strict;

use Class::Singleton;
use DateTime::TimeZone;
use DateTime::TimeZone::OlsonDB;

@DateTime::TimeZone::Asia::Kuala_Lumpur::ISA = ( 'Class::Singleton', 'DateTime::TimeZone' );

my $spans =
[
    [
DateTime::TimeZone::NEG_INFINITY,
59295517992,
DateTime::TimeZone::NEG_INFINITY,
59295542400,
24408,
0,
'LMT'
    ],
    [
59295517992,
60097482276,
59295542916,
60097507200,
24924,
0,
'SMT'
    ],
    [
60097482276,
60968048400,
60097507476,
60968073600,
25200,
0,
'MALT'
    ],
    [
60968048400,
61255932000,
60968074800,
61255958400,
26400,
0,
'MALT'
    ],
    [
61255932000,
61367814000,
61255964400,
61367846400,
32400,
0,
'JST'
    ],
    [
61367814000,
61504504800,
61367840400,
61504531200,
26400,
0,
'MALT'
    ],
    [
61504504800,
62524715400,
61504531800,
62524742400,
27000,
0,
'MALT'
    ],
    [
62524715400,
DateTime::TimeZone::INFINITY,
62524686600,
DateTime::TimeZone::INFINITY,
28800,
0,
'MYT'
    ],
];

sub _max_year { 2013 }

sub _new_instance
{
    return shift->_init( @_, spans => $spans );
}



1;

